/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include "baseqtversion.h"

namespace QtSupport {

class QTSUPPORT_EXPORT DesktopQtVersion : public BaseQtVersion
{
public:
    DesktopQtVersion();
    DesktopQtVersion(const Utils::FileName &path, bool isAutodetected = false, const QString &autodetectionSource = QString());
    DesktopQtVersion *clone() const;

    QString type() const;

    QStringList warningReason() const;

    QList<ProjectExplorer::Abi> detectQtAbis() const;

    QString description() const;

    QSet<Core::Id> availableFeatures() const;
    QSet<Core::Id> targetDeviceTypes() const;

    virtual void fromMap(const QVariantMap &map);

    QString qmlsceneCommand() const;

protected:
    DesktopQtVersion(const DesktopQtVersion &other);

private:

    enum TargetBinaries { QmlScene };
    QString findTargetBinary(TargetBinaries binary) const;

    mutable QString m_qmlsceneCommand;
};

} // QtSupport
