#include "myapp.h"


MyApp::MyApp(int &argc, char **argv)
    : IDS::NXT::VApp{argc, argv},
      _engine{_resultcollection}
{
    // at this point all result sources must be created.
}

void MyApp::imageAvailable(std::shared_ptr<IDS::NXT::Hardware::Image> image)
{
    Q_UNUSED(image);
}

void MyApp::abortVision()
{
    // implement me
}
