#ifndef NXTDEVICETESTER_H
#define NXTDEVICETESTER_H

#include <projectexplorer/devicesupport/idevice.h>

namespace Nxt {
namespace Internal {

class NxtDeviceTester : public ProjectExplorer::DeviceTester
{
    Q_OBJECT

public:
    explicit NxtDeviceTester(QObject *parent = 0);
    ~NxtDeviceTester();

    void testDevice(const ProjectExplorer::IDevice::ConstPtr &deviceConfiguration);
    void stopTest();

private:
    void setFinished(ProjectExplorer::DeviceTester::TestResult result);
};

} // namespace Internal
} // namespace Nxt

#endif // NXTDEVICETESTER_H
