#ifndef NXTRUNRESTCONFIGURATION_H
#define NXTRUNRESTCONFIGURATION_H

#include <remotelinux/remotelinuxrunconfiguration.h>

namespace Nxt {
namespace Internal {

class NxtRunRestConfiguration : public RemoteLinux::RemoteLinuxRunConfiguration
{
    Q_OBJECT
public:
    NxtRunRestConfiguration(ProjectExplorer::Target *parent);

#ifdef NXTRUN_RESTCONFIGURATION_HIDE_CONFIGURATION_WIDGET
    QWidget *createConfigurationWidget() override;
#endif

    ProjectExplorer::Runnable runnable() const override;

    //QString localExecutableFilePath() const;
    //QString defaultRemoteExecutableFilePath() const override;

    QString disabledReason() const override;

    QString vappName;
    QString targetName;
    QString projectName;
    QString remotePath;

protected:
    void updateEnabledState() override;
    bool fromMap(const QVariantMap &map) override;
private:
    QString defaultDisplayName() const;
    mutable QString m_disabledReason;
};

} // namespace Internal
} // namespace Nxt

#endif // NXTRUNRESTCONFIGURATION_H
