/****************************************************************************
**
** Copyright (C) 2016 Jochen Becher
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include "qmt/infrastructure/qmt_global.h"
#include <QSizeF>

namespace qmt {

class ShapeValueF;
class ShapePointF;
class ShapeSizeF;
class ShapeConstVisitor;

class QMT_EXPORT IconShape
{
    class IconShapePrivate;

public:
    IconShape();
    IconShape(const IconShape &rhs);
    ~IconShape();

    IconShape &operator=(const IconShape &rhs);

    QSizeF size() const;
    void setSize(const QSizeF &size);

    void addLine(const ShapePointF &pos1, const ShapePointF &pos2);
    void addRect(const ShapePointF &pos, const ShapeSizeF &size);
    void addRoundedRect(const ShapePointF &pos, const ShapeSizeF &size, const ShapeValueF &radius);
    void addCircle(const ShapePointF &center, const ShapeValueF &radius);
    void addEllipse(const ShapePointF &center, const ShapeSizeF &radius);
    void addDiamond(const ShapePointF &center, const ShapeSizeF &size, bool filled);
    void addTriangle(const ShapePointF &center, const ShapeSizeF &size, bool filled);
    void addArc(const ShapePointF &center, const ShapeSizeF &radius,
                qreal startAngle, qreal spanAngle);

    void moveTo(const ShapePointF &pos);
    void lineTo(const ShapePointF &pos);
    void arcMoveTo(const ShapePointF &center, const ShapeSizeF &radius, qreal angle);
    void arcTo(const ShapePointF &center, const ShapeSizeF &radius,
               qreal startAngle, qreal sweepLength);
    void closePath();

    void visitShapes(ShapeConstVisitor *visitor) const;

private:
    IconShapePrivate *d;
};

} // namespace qmt
