#ifndef MYPLUGIN_H
#define MYPLUGIN_H

// Include framework headers
#include "plugin.h"

/**
 * @brief The plugin-specific plugin object
 *
 * The plugin object serves as central object in the nxt plugin.
 */
class MyPlugin : public IDS::NXT::Plugin
{
    // This Qt macro is needed for the signal/slot mechanism
    Q_OBJECT

public:
    /**
     * @brief Constructor
     * @param argc count of command line arguments
     * @param argv list of command line arguments
     */
    MyPlugin(int &argc, char **argv);
};

#endif // MYPLUGIN_H
