/****************************************************************************
**
** Copyright (C) 2012 - 2014 Jolla Ltd.
** Contact: http://jolla.com/
**
** This file is part of Qt Creator.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** Other Usage
**
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Digia.
**
****************************************************************************/

#ifndef NXTSDKSELECTIONDIALOG_H
#define NXTSDKSELECTIONDIALOG_H

#include <QDialog>

namespace Nxt {
namespace Internal {

namespace Ui {
class NxtSdkSelectionDialog;
}

class NxtSdkSelectionDialog : public QDialog
{
    Q_OBJECT
public:
    explicit NxtSdkSelectionDialog(QWidget *parent = nullptr);
    ~NxtSdkSelectionDialog() override;

    QString selectedSdkName() const;

private slots:
    void handleItemSelectionChanged();
    void handleItemDoubleClicked();

private:
    Ui::NxtSdkSelectionDialog *m_ui;
};

} // namespace Internal
} // namespace Nxt

#endif // NXT_SDKSELECTIONDIALOG_H
