EXPORTS
DebugExtensionInitialize
DebugExtensionUninitialize
DebugExtensionNotify
pid
expandlocals
watches
locals
dumplocal
typecast
addsymbol
assign
threads
registers
modules
idle
setparameter
help
memory
expression
shutdownex
test
stack
qmlstack
addwatch
widgetat
breakpoints
KnownStructOutput
script
