/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#include "winrtphoneqtversion.h"

#include "winrtconstants.h"

#include <coreplugin/id.h>
#include <qtsupport/qtsupportconstants.h>

#include <QSet>

namespace WinRt {
namespace Internal {

WinRtPhoneQtVersion::WinRtPhoneQtVersion(const Utils::FileName &path, bool isAutodetected,
                                         const QString &autodetectionSource)
    : WinRtQtVersion(path, isAutodetected, autodetectionSource)
{
}

QString WinRtPhoneQtVersion::description() const
{
    return tr("Windows Phone");
}

QtSupport::BaseQtVersion *WinRtPhoneQtVersion::clone() const
{
    return new WinRtPhoneQtVersion(*this);
}

QString WinRtPhoneQtVersion::type() const
{
    return QLatin1String(Constants::WINRT_WINPHONEQT);
}

QSet<Core::Id> WinRtPhoneQtVersion::targetDeviceTypes() const
{
    return {Constants::WINRT_DEVICE_TYPE_PHONE, Constants::WINRT_DEVICE_TYPE_EMULATOR};
}

} // Internal
} // WinRt
