/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#include "clangstaticanalyzerprojectsettingsmanager.h"

#include "clangstaticanalyzerprojectsettings.h"

#include <projectexplorer/session.h>

namespace ClangStaticAnalyzer {
namespace Internal {

ProjectSettingsManager::ProjectSettingsManager()
{
    QObject::connect(ProjectExplorer::SessionManager::instance(),
            &ProjectExplorer::SessionManager::aboutToRemoveProject,
            &ProjectSettingsManager::handleProjectToBeRemoved);
}

ProjectSettings *ProjectSettingsManager::getSettings(ProjectExplorer::Project *project)
{
    auto &settings = m_settings[project];
    if (!settings)
        settings.reset(new ProjectSettings(project));
    return settings.data();
}

void ProjectSettingsManager::handleProjectToBeRemoved(ProjectExplorer::Project *project)
{
    m_settings.remove(project);
}

ProjectSettingsManager::SettingsMap ProjectSettingsManager::m_settings;

} // namespace Internal
} // namespace ClangStaticAnalyzer
