/****************************************************************************
**
** Copyright (C) 2012 - 2016 Jolla Ltd.
** Contact: http://jolla.com/
**
** This file is part of Qt Creator.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** Other Usage
**
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Digia.
**
****************************************************************************/

#ifndef NXTICONS_H
#define NXTICONS_H

#include <nxtconstants.h>

#include <utils/icon.h>

namespace Nxt {
namespace Icons {

const Utils::Icon NXT_MODE_ICON_CLASSIC((QLatin1String(Constants::NXT_OPTIONS_CATEGORY_ICON)));
const Utils::Icon NXT_MODE_ICON_FLAT({{QLatin1String(Constants::NXT_OPTIONS_CATEGORY_ICON_MASK), Utils::Theme::IconsBaseColor}});
const Utils::Icon NXT_MODE_ICON_FLAT_ACTIVE({{QLatin1String(Constants::NXT_OPTIONS_CATEGORY_ICON_MASK), Utils::Theme::IconsModeProjectActiveColor}});//todo//IconsModeNxtActiveColor

} // namespace Icons
} // namespace Nxt

#endif // NXTICONS_H
