/****************************************************************************
**
** Copyright (C) 2012 - 2017 Jolla Ltd.
** Contact: http://jolla.com/
**
** This file is part of Qt Creator.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** Other Usage
**
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Digia.
**
****************************************************************************/

#ifndef NXTDEVICEDEBUGSUPPORT_H
#define NXTDEVICEDEBUGSUPPORT_H

#ifdef HACK_BYPASS_PRIVATE_ENVIRONMENT_ACCESS /* bypass the private access to debugger */
#define setBreakOnMainNextTime setBreakOnMainNextTime(); Internal::DebuggerRunParameters& runParametersHack
#endif /* bypass the private access to debugger */

#include <debugger/debuggerruncontrol.h>

#ifdef HACK_BYPASS_PRIVATE_ENVIRONMENT_ACCESS /* bypass the private access to debugger */
#undef setBreakOnMainNextTime
#endif /* bypass the private access to debugger */

namespace ProjectExplorer {
    class RunControl;
}

namespace Nxt {
namespace Internal {

// Based on RemoteLinux::LinuxDeviceDebugSupport
class NxtDeviceDebugSupport : public Debugger::DebuggerRunTool
{
public:
    NxtDeviceDebugSupport(ProjectExplorer::RunControl *runControl, ProjectExplorer::Kit *kit = nullptr);

protected:
    void start() override;
};

} // namespace Internal
} // namespace Nxt

#endif // NXTDEVICEDEBUGSUPPORT_H
