/****************************************************************************
**
** Copyright (C) 2016 Dmitry Savchenko
** Copyright (C) 2016 Vasiliy Sorokin
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#include "todoitemsscanner.h"
#include "lineparser.h"

#include <projectexplorer/projectexplorer.h>
#include <projectexplorer/project.h>

namespace Todo {
namespace Internal {

TodoItemsScanner::TodoItemsScanner(const KeywordList &keywordList, QObject *parent) :
    QObject(parent), m_keywordList(keywordList)
{
}

void TodoItemsScanner::setParams(const KeywordList &keywordList)
{
    m_keywordList = keywordList;
    scannerParamsChanged();
}


// Descendants can use this to process comment lines
void TodoItemsScanner::processCommentLine(const QString &fileName, const QString &comment,
                                          unsigned lineNumber, QList<TodoItem> &outItemList)
{
    LineParser parser(m_keywordList);
    QList<TodoItem> newItemList = parser.parse(comment);

    for (int i = 0; i < newItemList.count(); ++i) {
        newItemList[i].line = lineNumber;
        newItemList[i].file = Utils::FileName::fromString(fileName);
    }

    outItemList << newItemList;
}

}
}
