/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

namespace Bookmarks {
namespace Constants {

const char BOOKMARKS_TOGGLE_ACTION[]      = "Bookmarks.Toggle";
const char BOOKMARKS_MOVEUP_ACTION[]      = "Bookmarks.MoveUp";
const char BOOKMARKS_MOVEDOWN_ACTION[]    = "Bookmarks.MoveDown";
const char BOOKMARKS_EDITNOTE_ACTION[]    = "Bookmarks.EditNote";
const char BOOKMARKS_PREV_ACTION[]        = "Bookmarks.Previous";
const char BOOKMARKS_NEXT_ACTION[]        = "Bookmarks.Next";
const char BOOKMARKS_PREVDIR_ACTION[]     = "Bookmarks.PreviousDirectory";
const char BOOKMARKS_NEXTDIR_ACTION[]     = "Bookmarks.NextDirectory";
const char BOOKMARKS_PREVDOC_ACTION[]     = "Bookmarks.PreviousDocument";
const char BOOKMARKS_NEXTDOC_ACTION[]     = "Bookmarks.NextDocument";
const char BOOKMARKS_TEXT_MARK_CATEGORY[] = "Bookmarks.TextMarkCategory";

const char BOOKMARKS_MENU[]               = "Bookmarks.Menu";
const char BOOKMARKS_CONTEXT[]            = "Bookmarks";

} // namespace Constants
} // namespace Bookmarks
