/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include "cpptools_global.h"

#include <QStringList>

#include <set>

namespace CppTools {

class CPPTOOLS_EXPORT FileIterationOrder {
public:
    struct Entry {
        Entry(const QString &filePath,
              const QString &projectPartId = QString(),
              int commonFilePathPrefixLength = 0,
              int commonProjectPartPrefixLength = 0);

        friend CPPTOOLS_EXPORT bool operator<(const Entry &first, const Entry &second);

        const QString filePath;
        const QString projectPartId;
        int commonFilePathPrefixLength = 0;
        int commonProjectPartPrefixLength = 0;
    };

    FileIterationOrder();
    FileIterationOrder(const QString &referenceFilePath,
                       const QString &referenceProjectPartId);

    void setReference(const QString &filePath, const QString &projectPartId);
    bool isValid() const;

    void insert(const QString &filePath, const QString &projectPartId = QString());
    void remove(const QString &filePath, const QString &projectPartId);
    QStringList toStringList() const;

private:
    Entry createEntryFromFilePath(const QString &filePath,
                                  const QString &projectPartId) const;

private:
    QString m_referenceFilePath;
    QString m_referenceProjectPartId;
    std::multiset<Entry> m_set;
};

CPPTOOLS_EXPORT bool operator<(const FileIterationOrder::Entry &first,
                               const FileIterationOrder::Entry &second);

} // namespace CppTools
