/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include "autotestconstants.h"

#include <QSettings>

namespace Autotest {
namespace Internal {

class IFrameworkSettings
{
public:
    IFrameworkSettings() {}
    virtual ~IFrameworkSettings() {}

    virtual QString name() const = 0;

    void toSettings(QSettings *s) const
    {
        s->beginGroup(Constants::SETTINGSGROUP);
        s->beginGroup(name());
        toFrameworkSettings(s);
        s->endGroup();
        s->endGroup();
    }

    void fromSettings(QSettings *s)
    {
        s->beginGroup(Constants::SETTINGSGROUP);
        s->beginGroup(name());
        fromFrameworkSettings(s);
        s->endGroup();
        s->endGroup();
    }

protected:
    virtual void toFrameworkSettings(QSettings *s) const = 0;
    virtual void fromFrameworkSettings(const QSettings *s) = 0;
};

} // namespace Internal
} // namespace Autotest
