#ifndef NXTABSTRACTBUILDSTEP_H
#define NXTABSTRACTBUILDSTEP_H

#include <projectexplorer/buildstep.h>
#include <projectexplorer/processparameters.h>
#include <projectexplorer/abstractprocessstep.h>

namespace Nxt {
namespace Internal {

class NxtAbstractBuildStep : public ProjectExplorer::AbstractProcessStep//BuildStep
{
    Q_OBJECT

public:
    explicit NxtAbstractBuildStep(ProjectExplorer::BuildStepList *bsl, Core::Id id);

    bool init(QList<const BuildStep *> &earlierSteps) override;
    void run(QFutureInterface<bool> &fi) override;

    ProjectExplorer::BuildStepConfigWidget *createConfigWidget() override;
    bool immutable() const override;
    //bool runInGuiThread() const override;
    //ProjectExplorer::ProcessParameters *processParameters() { return &m_param; }

protected:
    //ProjectExplorer::ProcessParameters m_param;
};

} // namespace Internal
} // namespace Nxt

#endif // NXTABSTRACTBUILDSTEP_H
