/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include "projectexplorer_export.h"

#include <utils/icon.h>

namespace ProjectExplorer {
namespace Icons {

PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILD;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILD_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILD_SMALL;
PROJECTEXPLORER_EXPORT extern const Utils::Icon REBUILD;
PROJECTEXPLORER_EXPORT extern const Utils::Icon RUN;
PROJECTEXPLORER_EXPORT extern const Utils::Icon RUN_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon WINDOW;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_READY_INDICATOR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_READY_INDICATOR_OVERLAY;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_CONNECTED_INDICATOR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_CONNECTED_INDICATOR_OVERLAY;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_DISCONNECTED_INDICATOR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_DISCONNECTED_INDICATOR_OVERLAY;

PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START_SMALL;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START_SMALL_TOOLBAR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon ANALYZER_START_SMALL;
PROJECTEXPLORER_EXPORT extern const Utils::Icon ANALYZER_START_SMALL_TOOLBAR;

PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_MOVEUP;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_MOVEDOWN;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_DISABLE;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_REMOVE;

PROJECTEXPLORER_EXPORT extern const Utils::Icon DESKTOP_DEVICE;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DESKTOP_DEVICE_SMALL;

PROJECTEXPLORER_EXPORT extern const Utils::Icon MODE_PROJECT_CLASSIC;
PROJECTEXPLORER_EXPORT extern const Utils::Icon MODE_PROJECT_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon MODE_PROJECT_FLAT_ACTIVE;

} // namespace Icons
} // namespace ProjectExplorer
