QFrame
{
    background-color: creatorTheme.QmlDesignerBackgroundColorDarkAlternate;
    color: creatorTheme.PanelTextColorLight;
    font-size: 11px;
    border-radius: 0px;
}

QTableView {
    color: creatorTheme.PanelTextColorLight;
    selection-color: creatorTheme.PanelTextColorLight;
    selection-background-color: creatorTheme.QmlDesigner_HighlightColor;

}

QTabBar QToolButton {
    background-color: creatorTheme.QmlDesignerBackgroundColorDarkAlternate;
    border: 1px solid creatorTheme.QmlDesignerBackgroundColorDarker;
    border-radius: 0px;
}

QTableView::item
{
    border: 0px;
    padding-left: 4px;
}

QTableView::item:focus
{
    border: none;
    background-color: transparent;
}

QTableView::item:selected
{
    border: none
}

QHeaderView::section {
    background-color: #494949;
    padding: 4px;
    border: 1px solid black;
    color: #cacaca;
    margin: 2px
}

QTableView {
    alternate-background-color: #414141;
}

QWidget#widgetSpacer {
    background-color: creatorTheme.QmlDesignerTabLight;
}

QStackedWidget {
    border: 0px;
    background-color: #4f4f4f;
}

QTabBar::tab:selected {
    border: none;
    border-image: none;
    image: none;

    background-color: creatorTheme.QmlDesignerTabLight;
    color: creatorTheme.QmlDesignerTabDark;
}


QTabBar::tab {
    width: 88px;
    height: 22px;
    margin-top: 0x;
    margin-bottom: 0px;
    margin-left: 0px;
    margin-right: 0px;
    font: bold;
    font-size: 11px;
    background-color: creatorTheme.QmlDesignerTabDark;
    color: creatorTheme.QmlDesignerTabLight;
}

QSpinBox
{
    font-size: 11px;
    color: white;
    padding-right: 2px;
    padding-top: 2px;
    padding-bottom: 2px;
    padding-left: 12px;
    border: 2px solid #0F0F0F;
    border-width: 1;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
    stop: 0 #2c2c2c, stop: 1 #333333);

     min-height: 22px;
}

 QDoubleSpinBox
 {
    font-size: 11px;
    color: white;
    padding-right: 2px;
    padding-top: 2px;
    padding-bottom: 2px;
    padding-left: 12px;
    border: 2px solid #0F0F0F;
    border-width: 1;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
    stop: 0 #2c2c2c, stop: 1 #333333);
    min-height: 22px;
 }

QLineEdit
{
    color: white;
    font-size: 11px;
    border: 2px solid #0F0F0F;
    border-width: 1;
    min-height: 26px;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
    stop: 0 #2c2c2c, stop: 1 #333333);
}
