/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#include "highlighterutils.h"
#include "generichighlighter/highlighter.h"
#include "generichighlighter/highlightdefinition.h"
#include "generichighlighter/manager.h"
#include <coreplugin/icore.h>
#include <utils/mimetypes/mimetype.h>

using namespace TextEditor;
using namespace Internal;

void TextEditor::setMimeTypeForHighlighter(Highlighter *highlighter, const Utils::MimeType &mimeType,
                                           const QString &filePath, QString *foundDefinitionId)
{
    QString definitionId = Manager::instance()->definitionIdByMimeTypeAndFile(mimeType, filePath);
    if (!definitionId.isEmpty()) {
        const QSharedPointer<HighlightDefinition> &definition =
            Manager::instance()->definition(definitionId);
        if (!definition.isNull() && definition->isValid())
            highlighter->setDefaultContext(definition->initialContext());
    }

    if (foundDefinitionId)
        *foundDefinitionId = definitionId;
}

