/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#include "clangquerytexteditorwidget.h"

#include "clangqueryhighlighter.h"
#include "clangqueryhoverhandler.h"

#include <texteditor/textdocument.h>

namespace ClangRefactoring {

ClangQueryTextEditorWidget::ClangQueryTextEditorWidget(QWidget *parent)
    : BaseClangQueryTextEditorWidget(parent),
      m_syntaxHighlighter(new ClangQueryHighlighter),
      m_hoverHandler(std::make_unique<ClangQueryHoverHandler>(m_syntaxHighlighter))
{
    textDocument()->setSyntaxHighlighter(m_syntaxHighlighter);
    textDocument()->setPlainText("functionDecl()");

    addHoverHandler(m_hoverHandler.get());
}

ClangQueryTextEditorWidget::~ClangQueryTextEditorWidget() = default;

ClangQueryHighlighter *ClangQueryTextEditorWidget::syntaxHighlighter() const
{
    return m_syntaxHighlighter;
}

} // namespace ClangRefactoring
