/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

#include "gtesttreeitem.h"

#include <cplusplus/ASTVisitor.h>
#include <cplusplus/CppDocument.h>
#include <cplusplus/Overview.h>

#include <QMap>

namespace Autotest {
namespace Internal {

inline bool operator<(const GTestCaseSpec &spec1, const GTestCaseSpec &spec2)
{
    if (spec1.testCaseName != spec2.testCaseName)
        return spec1.testCaseName < spec2.testCaseName;
    if (spec1.parameterized == spec2.parameterized) {
        if (spec1.typed == spec2.typed) {
            if (spec1.disabled == spec2.disabled)
                return false;
            else
                return !spec1.disabled;
        } else {
            return !spec1.typed;
        }
    } else {
        return !spec1.parameterized;
    }
}

class GTestVisitor : public CPlusPlus::ASTVisitor
{
public:
    explicit GTestVisitor(CPlusPlus::Document::Ptr doc);
    bool visit(CPlusPlus::FunctionDefinitionAST *ast);

    QMap<GTestCaseSpec, GTestCodeLocationList> gtestFunctions() const { return m_gtestFunctions; }

private:
    QString enclosingNamespaces(CPlusPlus::Symbol *symbol) const;

    CPlusPlus::Document::Ptr m_document;
    CPlusPlus::Overview m_overview;
    QMap<GTestCaseSpec, GTestCodeLocationList> m_gtestFunctions;
};

} // namespace Internal
} // namespace Autotest
