#ifndef GDBCOMMAND_H
#define GDBCOMMAND_H

#include "../command.h"

class GdbCommand : public Command
{
public:
    GdbCommand();
    QString name() const override;
    int execute() override;
    bool isValid() const override;
};

#endif // GDBCOMMAND_H
