/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#include "mockuptypecontainer.h"

#include <QDebug>

namespace QmlDesigner {

QmlDesigner::MockupTypeContainer::MockupTypeContainer()
{

}

QmlDesigner::MockupTypeContainer::MockupTypeContainer(const QmlDesigner::TypeName &type,
                                                      const QString &importUri,
                                                      int majorVersion,
                                                      int minorVersion, bool isItem)
    : m_typeName(type)
    ,m_importUri(importUri)
    ,m_majorVersion(majorVersion)
    ,m_minorVersion(minorVersion)
    ,m_isItem(isItem)
{

}

QString MockupTypeContainer::importUri() const
{
    return m_importUri;
}

QDataStream &operator<<(QDataStream &out, const MockupTypeContainer &container)
{
    out << container.typeName();
    out << container.importUri();
    out << container.majorVersion();
    out << container.minorVersion();
    out << container.isItem();

    return out;
}

QDataStream &operator>>(QDataStream &in, MockupTypeContainer &container)
{
    in >> container.m_typeName;
    in >> container.m_importUri;
    in >> container.m_majorVersion;
    in >> container.m_minorVersion;
    in >> container.m_isItem;

    return in;
}

QDebug operator <<(QDebug debug, const MockupTypeContainer &container)
{
    return debug.nospace() << "MockupTypeContainer("
                           << "typeName: " << container.typeName() << ", "
                           << "importUri: " << container.importUri() << ", "
                           << "majorVersion: " << container.majorVersion() << ", "
                           << "minorVersion: " << container.minorVersion() << ", "
                           << "isItem: " << container.isItem() << ")";
}


} // namespace QmlDesigner
