/****************************************************************************
**
** Copyright (C) 2016 Nicolas Arnaud-Cormos
** Contact: https://www.qt.io/licensing/
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#pragma once

namespace Macros {
namespace Constants {

const char M_TOOLS_MACRO[] = "Macros.Tools.Menu";

const char START_MACRO[] = "Macros.StartMacro";
const char END_MACRO[] = "Macros.EndMacro";
const char EXECUTE_LAST_MACRO[] = "Macros.ExecuteLastMacro";
const char SAVE_LAST_MACRO[] = "Macros.SaveLastMacro";
const char PREFIX_MACRO[] = "Macros.";

const char M_OPTIONS_PAGE[] = "Macros";
const char M_OPTIONS_TR_PAGE[] = QT_TRANSLATE_NOOP("Macros", "Macros");

const char M_STATUS_BUFFER[] = "Macros.Status";

const char M_EXTENSION[] = "mac";

} // namespace Constants
} // namespace Macros
