/****************************************************************************
**
** Copyright (C) 2012 - 2014 Jolla Ltd.
** Contact: http://jolla.com/
**
** This file is part of Qt Creator.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** Other Usage
**
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Digia.
**
****************************************************************************/

#include "nxtmode.h"

#include <nxtconstants.h>
#include <nxticons.h>

#include <coreplugin/modemanager.h>
#include <utils/icon.h>

#include <QDebug>

using namespace Core;

namespace Nxt {
namespace Internal {

NxtMode::NxtMode()
{
    setWidget(new QWidget);
    setContext(Context("Nxt.NxtMode"));
    setDisplayName(tr("IDS NXT"));
    setIcon(Utils::Icon::modeIcon(Icons::NXT_MODE_ICON_CLASSIC, Icons::NXT_MODE_ICON_FLAT, Icons::NXT_MODE_ICON_FLAT_ACTIVE));
    setPriority(80); // between "Projects" and "Analyze" modes
    setId("Nxt.NxtMode");
    setContextHelpId(QString());
    connect(ModeManager::instance(), &ModeManager::currentModeChanged, this, &NxtMode::handleUpdateContext);
}

void NxtMode::handleUpdateContext(Id newMode, Id oldMode)
{
    Q_UNUSED(newMode);
    Q_UNUSED(oldMode);
}

} // namespace Internal
} // namespace Nxt
